/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.activiti.ext;

import java.util.Map;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import com.activiti.api.boot.BootstrapConfigurer;

@Component(MultiBootstrapConfigurer.BEAN_NAME)
@Primary
public class MultiBootstrapConfigurer implements BootstrapConfigurer {
	
	public static final String BEAN_NAME = "inteligr8.multiext.bootstrapConfigurer";
    
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    
    @Override
    public void applicationContextInitialized(ApplicationContext applicationContext) {
    	this.logger.info("Multi-Ext enabled for BootstrapConfigurer");
    	
        Map<String, BootstrapConfigurer> beans = applicationContext.getBeansOfType(BootstrapConfigurer.class);
        beans.remove(BEAN_NAME);
        
        this.logger.debug("Executing {} bootstrap configurers", beans.size());
        for (Entry<String, BootstrapConfigurer> bootstrap : beans.entrySet()) {
            this.logger.trace("Executing bootstrap configurer: {}: {}", bootstrap.getKey(), bootstrap.getValue().getClass());
            bootstrap.getValue().applicationContextInitialized(applicationContext);
        }
    }

}
