/*
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.inteligr8.activiti.ext;

import java.util.Map;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Primary;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Component;

import com.activiti.api.security.AlfrescoSecurityConfigOverride;

@Component(MultiSecurityConfigOverride.BEAN_NAME)
@Primary
@ConditionalOnProperty("inteligr8.ext.securityConfigOverride.enabled")
public class MultiSecurityConfigOverride implements AlfrescoSecurityConfigOverride {
    
	public static final String BEAN_NAME = "inteligr8.multiext.securityConfigOverride";
	
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    
    @Autowired
    private ApplicationContext applicationContext;
    
    @Override
    public void configureGlobal(AuthenticationManagerBuilder auth, UserDetailsService userDetailsService) {
    	this.logger.info("Multi-Ext enabled for AlfrescoSecurityConfigOverride");
    	
        Map<String, AlfrescoSecurityConfigOverride> beans = this.applicationContext.getBeansOfType(AlfrescoSecurityConfigOverride.class);
        beans.remove(BEAN_NAME);
        
        this.logger.debug("Executing {} security config override", beans.size());
        for (Entry<String, AlfrescoSecurityConfigOverride> override : beans.entrySet()) {
            this.logger.trace("Executing security config override: {}: {}", override.getKey(), override.getValue().getClass());
            override.getValue().configureGlobal(auth, userDetailsService);
        }
    }

}
